"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchSintegra = void 0;
const imageManipulation_1 = require("../helpers/imageManipulation");
const ocr_1 = require("../helpers/ocr");
const puppeteerHelper_1 = require("../helpers/puppeteerHelper");
const setupPuppeteer_1 = require("../helpers/setupPuppeteer");
const estados_1 = __importDefault(require("../utils/estados"));
const utils_1 = require("../utils/utils");
const NEXT_PAGE_CODE_SELECTOR = "input[name='_dataPesquisa']";
const INPUT_SELECTOR = "input#cnpj";
const CAPTCHA_IMG_SELECTOR = "img#captchaImagem";
const ERROR_LABEL_SELECTOR = "div.alert-error";
const CAPTCHA_INPUT_SELECTOR = "input#captcha";
const FETCH_RESULT_BUTTON_SELECTOR = "button#envia";
const colorsToOverwrite = [
    { targetColor: "#00FFFF", newColor: "#ffffff" },
    { targetColor: "#00FF00", newColor: "#ffffff" },
];
async function fetchSintegra(cnpj) {
    const browser = await setupPuppeteer_1.setupBrowser();
    try {
        const page = await setupPuppeteer_1.setupPage(estados_1.default.MATO_GROSSO_DO_SUL.url, browser);
        await setupInitialPart(page, cnpj);
        await processCaptcha(page);
        let text = await getErrorLabelText(page);
        while ((text === null || text === void 0 ? void 0 : text.trim().includes("O Texto digitado não confere com a Imagem")) || (text === null || text === void 0 ? void 0 : text.trim().includes("Atenção: CNPJ inválido"))) {
            await page.reload({ waitUntil: "networkidle0" });
            await setupInitialPart(page, cnpj);
            await processCaptcha(page);
            text = await getErrorLabelText(page);
        }
        if ((text === null || text === void 0 ? void 0 : text.trim()) === "Atenção: CNPJ informado não contém IEs CCI" ||
            (text === null || text === void 0 ? void 0 : text.trim()) === "Atenção: Nenhum resultado encontrado") {
            throw new Error("Registro não existe");
        }
        await page.waitFor(NEXT_PAGE_CODE_SELECTOR, { hidden: true });
        const sintegra = await retrieveAndProcessValues(page);
        return buildData(sintegra);
    }
    catch (err) {
        throw err;
    }
    finally {
        await browser.close();
    }
}
exports.fetchSintegra = fetchSintegra;
async function setupInitialPart(page, cnpj) {
    const input = await page.waitFor(INPUT_SELECTOR, { visible: true });
    await input.type(cnpj.charAt(0));
    await input.type(cnpj);
}
async function getErrorLabelText(page) {
    return await puppeteerHelper_1.extractText(page, ERROR_LABEL_SELECTOR, { timeout: 15000 });
}
async function processCaptcha(page) {
    const text = await processImage(page);
    await submitResult(page, text);
}
async function processImage(page) {
    await page.waitFor(CAPTCHA_IMG_SELECTOR);
    const imgUrl = await page.$eval(CAPTCHA_IMG_SELECTOR, (node) => node === null || node === void 0 ? void 0 : node.src);
    const jimp = await imageManipulation_1.clearImage(imgUrl.toString());
    const clearedImg = await imageManipulation_1.replaceColorsFromImage(await imageManipulation_1.getImageBuffer(jimp), colorsToOverwrite);
    const img = await imageManipulation_1.applyFilters(await imageManipulation_1.getImageBuffer(clearedImg));
    return await ocr_1.processOcr(img, "0123456789");
}
async function submitResult(page, text) {
    await page.type(CAPTCHA_INPUT_SELECTOR, ocr_1.formatOCRResult(text));
    await page.click(FETCH_RESULT_BUTTON_SELECTOR);
}
async function retrieveAndProcessValues(page) {
    const ie = await puppeteerHelper_1.extractValue(page, "input#inscricaoEstadual");
    const dataDeInicioDeAtividade = await puppeteerHelper_1.extractValue(page, "input#dataInicioAtividade");
    const cnpj = await puppeteerHelper_1.extractValue(page, "input#pessoa_cnpjcpf");
    const razaoSocial = await puppeteerHelper_1.extractValue(page, "input#pessoa_nome");
    const descricaoDaAtividade = await puppeteerHelper_1.extractValue(page, "input#caeDescricao");
    const logradouro = await puppeteerHelper_1.extractValue(page, "input#pessoa_pessoaEndereco_logradouro");
    const numero = await puppeteerHelper_1.extractValue(page, "input#pessoa_pessoaEndereco_numero");
    const complemento = await puppeteerHelper_1.extractValue(page, "input#pessoa_pessoaEndereco_complemento");
    const cep = await puppeteerHelper_1.extractValue(page, "input#pessoa_pessoaEndereco_cep");
    const bairro = await puppeteerHelper_1.extractValue(page, "input#pessoa_pessoaEndereco_bairro");
    const municipio = await puppeteerHelper_1.extractValue(page, "input#pessoa_pessoaEndereco_municipio");
    const uf = await puppeteerHelper_1.extractValue(page, "input#pessoa_pessoaEndereco_uf");
    const situacaoCadastral = await puppeteerHelper_1.extractValue(page, "input#situacaoCadastral");
    const dataDaUltimaAtualizacao = await puppeteerHelper_1.extractValue(page, "input#dataAtualizacao");
    const motivoDaSituacao = await puppeteerHelper_1.extractValue(page, "input#descricaoSituacao");
    return {
        ie,
        dataDeInicioDeAtividade,
        cnpj,
        razaoSocial,
        descricaoDaAtividade,
        logradouro,
        numero,
        complemento,
        cep,
        bairro,
        municipio,
        uf,
        situacaoCadastral,
        dataDaUltimaAtualizacao,
        motivoDaSituacao,
    };
}
function buildData(data) {
    if (utils_1.isObjectEmpty(data)) {
        throw new Error("Sem dados");
    }
    return {
        ie: data.ie,
        dataDeInicioDeAtividade: data.dataDeInicioDeAtividade,
        cnpj: data.cnpj,
        razaoSocial: data.razaoSocial,
        descricaoDaAtividade: data.descricaoDaAtividade,
        logradouro: data.logradouro,
        numero: data.numero,
        complemento: data.complemento,
        cep: data.cep,
        bairro: data.bairro,
        municipio: data.municipio,
        uf: data.uf,
        situacaoCadastral: data.situacaoCadastral,
        dataDaUltimaAtualizacao: data.dataDaUltimaAtualizacao,
        motivoDaSituacao: data.motivoDaSituacao,
    };
}
//# sourceMappingURL=sintegraMS.js.map